@@ -274,7 +274,7 @@ type Options struct {
 	WriteToStdout bool
 
 	OmitRuntimeForTests     bool
-	UnusedImportsTS         UnusedImportsTS
+	UnusedImportFlagsTS     UnusedImportFlagsTS
 	UseDefineForClassFields MaybeBool
 	ASCIIOnly               bool
 	KeepNames               bool
@@ -303,27 +303,49 @@ const (
 	TargetWasConfiguredAndAtLeastES2022
 )
 
-type UnusedImportsTS uint8
+type UnusedImportFlagsTS uint8
 
+// With !UnusedImportKeepStmt && !UnusedImportKeepValues:
+//
+//   "import 'foo'"                      => "import 'foo'"
+//   "import * as unused from 'foo'"     => ""
+//   "import { unused } from 'foo'"      => ""
+//   "import { type unused } from 'foo'" => ""
+//
+// With UnusedImportKeepStmt && !UnusedImportKeepValues:
+//
+//   "import 'foo'"                      => "import 'foo'"
+//   "import * as unused from 'foo'"     => "import 'foo'"
+//   "import { unused } from 'foo'"      => "import 'foo'"
+//   "import { type unused } from 'foo'" => "import 'foo'"
+//
+// With !UnusedImportKeepStmt && UnusedImportKeepValues:
+//
+//   "import 'foo'"                      => "import 'foo'"
+//   "import * as unused from 'foo'"     => "import * as unused from 'foo'"
+//   "import { unused } from 'foo'"      => "import { unused } from 'foo'"
+//   "import { type unused } from 'foo'" => ""
+//
+// With UnusedImportKeepStmt && UnusedImportKeepValues:
+//
+//   "import 'foo'"                      => "import 'foo'"
+//   "import * as unused from 'foo'"     => "import * as unused from 'foo'"
+//   "import { unused } from 'foo'"      => "import { unused } from 'foo'"
+//   "import { type unused } from 'foo'" => "import {} from 'foo'"
+//
 const (
-	// "import { unused } from 'foo'" => "" (TypeScript's default behavior)
-	UnusedImportsRemoveStmt UnusedImportsTS = iota
-
-	// "import { unused } from 'foo'" => "import 'foo'" ("importsNotUsedAsValues" != "remove")
-	UnusedImportsKeepStmtRemoveValues
-
-	// "import { unused } from 'foo'" => "import { unused } from 'foo'" ("preserveValueImports" == true)
-	UnusedImportsKeepValues
+	UnusedImportKeepStmt   UnusedImportFlagsTS = 1 << iota // "importsNotUsedAsValues" != "remove"
+	UnusedImportKeepValues                                 // "preserveValueImports" == true
 )
 
-func UnusedImportsFromTsconfigValues(preserveImportsNotUsedAsValues bool, preserveValueImports bool) UnusedImportsTS {
+func UnusedImportFlagsFromTsconfigValues(preserveImportsNotUsedAsValues bool, preserveValueImports bool) (flags UnusedImportFlagsTS) {
 	if preserveValueImports {
-		return UnusedImportsKeepValues
+		flags |= UnusedImportKeepValues
 	}
 	if preserveImportsNotUsedAsValues {
-		return UnusedImportsKeepStmtRemoveValues
+		flags |= UnusedImportKeepStmt
 	}
-	return UnusedImportsRemoveStmt
+	return
 }
 
 type TSTarget struct {
