@@ -5,6 +5,7 @@ import (
 	"strings"
 	"sync"
 
+	"github.com/evanw/esbuild/internal/helpers"
 	"github.com/evanw/esbuild/internal/js_ast"
 	"github.com/evanw/esbuild/internal/logger"
 )
@@ -78,6 +79,22 @@ var knownGlobals = [][]string{
 	{"Object", "prototype", "valueOf"},
 	{"Object", "prototype", "watch"},
 
+	// Symbol: Static properties
+	// https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Symbol#static_properties
+	{"Symbol", "asyncIterator"},
+	{"Symbol", "hasInstance"},
+	{"Symbol", "isConcatSpreadable"},
+	{"Symbol", "iterator"},
+	{"Symbol", "match"},
+	{"Symbol", "matchAll"},
+	{"Symbol", "replace"},
+	{"Symbol", "search"},
+	{"Symbol", "species"},
+	{"Symbol", "split"},
+	{"Symbol", "toPrimitive"},
+	{"Symbol", "toStringTag"},
+	{"Symbol", "unscopables"},
+
 	// Math: Static properties
 	// https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Math#Static_properties
 	{"Math", "E"},
@@ -937,7 +954,7 @@ func ProcessDefines(userDefines map[string]DefineData) ProcessedDefines {
 
 		// Try to merge with existing dot defines first
 		for i, define := range dotDefines {
-			if arePartsEqual(parts, define.Parts) {
+			if helpers.StringArraysEqual(parts, define.Parts) {
 				define := &dotDefines[i]
 				define.Data = mergeDefineData(define.Data, data)
 				found = true
@@ -961,15 +978,3 @@ func ProcessDefines(userDefines map[string]DefineData) ProcessedDefines {
 	}
 	return result
 }
-
-func arePartsEqual(a []string, b []string) bool {
-	if len(a) != len(b) {
-		return false
-	}
-	for i := range a {
-		if a[i] != b[i] {
-			return false
-		}
-	}
-	return true
-}
