@@ -113,6 +113,10 @@ func (p *parser) eat(kind css_lexer.T) bool {
 }
 
 func (p *parser) expect(kind css_lexer.T) bool {
+	return p.expectWithMatchingLoc(kind, logger.Loc{Start: -1})
+}
+
+func (p *parser) expectWithMatchingLoc(kind css_lexer.T, matchingLoc logger.Loc) bool {
 	if p.eat(kind) {
 		return true
 	}
@@ -123,6 +127,7 @@ func (p *parser) expect(kind css_lexer.T) bool {
 
 	var text string
 	var suggestion string
+	var notes []logger.MsgData
 
 	expected := kind.String()
 	if strings.HasPrefix(expected, "\"") && strings.HasSuffix(expected, "\"") {
@@ -133,6 +138,11 @@ func (p *parser) expect(kind css_lexer.T) bool {
 		// Have a nice error message for forgetting a trailing semicolon or colon
 		text = fmt.Sprintf("Expected %s", expected)
 		t = p.at(p.index - 1)
+	} else if (kind == css_lexer.TCloseBrace || kind == css_lexer.TCloseBracket || kind == css_lexer.TCloseParen) && matchingLoc.Start != -1 {
+		// Have a nice error message for forgetting a closing brace/bracket/parenthesis
+		c := p.source.Contents[matchingLoc.Start : matchingLoc.Start+1]
+		text = fmt.Sprintf("Expected %s to go with %q", expected, c)
+		notes = append(notes, p.tracker.MsgData(logger.Range{Loc: matchingLoc, Len: 1}, fmt.Sprintf("The unbalanced %q is here:", c)))
 	} else {
 		switch t.Kind {
 		case css_lexer.TEndOfFile, css_lexer.TWhitespace:
@@ -148,7 +158,9 @@ func (p *parser) expect(kind css_lexer.T) bool {
 	if t.Range.Loc.Start > p.prevError.Start {
 		data := p.tracker.MsgData(t.Range, text)
 		data.Location.Suggestion = suggestion
-		p.log.AddMsg(logger.Msg{Kind: logger.Warning, Data: data})
+		if override, ok := logger.AllowOverride(p.log.Overrides, logger.MsgID_CSS_CSSSyntaxError, logger.Warning); ok {
+			p.log.AddMsg(logger.Msg{Kind: override, Data: data, Notes: notes})
+		}
 		p.prevError = t.Range.Loc
 	}
 	return false
@@ -166,7 +178,7 @@ func (p *parser) unexpected() {
 		default:
 			text = fmt.Sprintf("Unexpected %q", p.raw())
 		}
-		p.log.Add(logger.Warning, &p.tracker, t.Range, text)
+		p.log.AddID(logger.MsgID_CSS_CSSSyntaxError, logger.Warning, &p.tracker, t.Range, text)
 		p.prevError = t.Range.Loc
 	}
 }
@@ -594,10 +606,11 @@ func (p *parser) parseURLOrString() (string, logger.Range, bool) {
 
 	case css_lexer.TFunction:
 		if p.decoded() == "url" {
+			matchingLoc := logger.Loc{Start: p.current().Range.End() - 1}
 			p.advance()
 			t = p.current()
 			text := p.decoded()
-			if p.expect(css_lexer.TString) && p.expect(css_lexer.TCloseParen) {
+			if p.expect(css_lexer.TString) && p.expectWithMatchingLoc(css_lexer.TCloseParen, matchingLoc) {
 				return text, t.Range, true
 			}
 		}
@@ -737,10 +750,11 @@ abortRuleParser:
 	case "charset":
 		switch context.charsetValidity {
 		case atRuleInvalid:
-			p.log.Add(logger.Warning, &p.tracker, atRange, "\"@charset\" must be the first rule in the file")
+			p.log.AddID(logger.MsgID_CSS_InvalidAtCharset, logger.Warning, &p.tracker, atRange, "\"@charset\" must be the first rule in the file")
 
 		case atRuleInvalidAfter:
-			p.log.AddWithNotes(logger.Warning, &p.tracker, atRange, "\"@charset\" must be the first rule in the file",
+			p.log.AddIDWithNotes(logger.MsgID_CSS_InvalidAtCharset, logger.Warning, &p.tracker, atRange,
+				"\"@charset\" must be the first rule in the file",
 				[]logger.MsgData{p.tracker.MsgData(logger.Range{Loc: context.afterLoc},
 					"This rule cannot come before a \"@charset\" rule")})
 
@@ -750,7 +764,7 @@ abortRuleParser:
 			if p.peek(css_lexer.TString) {
 				encoding := p.decoded()
 				if !strings.EqualFold(encoding, "UTF-8") {
-					p.log.Add(logger.Warning, &p.tracker, p.current().Range,
+					p.log.AddID(logger.MsgID_CSS_UnsupportedAtCharset, logger.Warning, &p.tracker, p.current().Range,
 						fmt.Sprintf("\"UTF-8\" will be used instead of unsupported charset %q", encoding))
 				}
 				p.advance()
@@ -763,10 +777,11 @@ abortRuleParser:
 	case "import":
 		switch context.importValidity {
 		case atRuleInvalid:
-			p.log.Add(logger.Warning, &p.tracker, atRange, "\"@import\" is only valid at the top level")
+			p.log.AddID(logger.MsgID_CSS_InvalidAtImport, logger.Warning, &p.tracker, atRange, "\"@import\" is only valid at the top level")
 
 		case atRuleInvalidAfter:
-			p.log.AddWithNotes(logger.Warning, &p.tracker, atRange, "All \"@import\" rules must come first",
+			p.log.AddIDWithNotes(logger.MsgID_CSS_InvalidAtImport, logger.Warning, &p.tracker, atRange,
+				"All \"@import\" rules must come first",
 				[]logger.MsgData{p.tracker.MsgData(logger.Range{Loc: context.afterLoc},
 					"This rule cannot come before an \"@import\" rule")})
 
@@ -829,6 +844,7 @@ abortRuleParser:
 		p.eat(css_lexer.TWhitespace)
 		blockStart := p.index
 
+		matchingLoc := p.current().Range.Loc
 		if p.expect(css_lexer.TOpenBrace) {
 			var blocks []css_ast.KeyframeBlock
 
@@ -866,7 +882,18 @@ abortRuleParser:
 							continue
 
 						case css_lexer.TOpenBrace:
+							blockMatchingLoc := p.current().Range.Loc
 							p.advance()
+							rules := p.parseListOfDeclarations()
+							p.expectWithMatchingLoc(css_lexer.TCloseBrace, blockMatchingLoc)
+
+							// "@keyframes { from {} to { color: red } }" => "@keyframes { to { color: red } }"
+							if !p.options.MinifySyntax || len(rules) > 0 {
+								blocks = append(blocks, css_ast.KeyframeBlock{
+									Selectors: selectors,
+									Rules:     rules,
+								})
+							}
 							break selectors
 
 						case css_lexer.TCloseBrace, css_lexer.TEndOfFile:
@@ -907,17 +934,6 @@ abortRuleParser:
 							break badSyntax
 						}
 					}
-
-					rules := p.parseListOfDeclarations()
-					p.expect(css_lexer.TCloseBrace)
-
-					// "@keyframes { from {} to { color: red } }" => "@keyframes { to { color: red } }"
-					if !p.options.MinifySyntax || len(rules) > 0 {
-						blocks = append(blocks, css_ast.KeyframeBlock{
-							Selectors: selectors,
-							Rules:     rules,
-						})
-					}
 				}
 			}
 
@@ -925,7 +941,7 @@ abortRuleParser:
 			for !p.peek(css_lexer.TCloseBrace) && !p.peek(css_lexer.TEndOfFile) {
 				p.parseComponentValue()
 			}
-			p.expect(css_lexer.TCloseBrace)
+			p.expectWithMatchingLoc(css_lexer.TCloseBrace, matchingLoc)
 			prelude := p.convertTokens(p.tokens[preludeStart:blockStart])
 			block, _ := p.convertTokensHelper(p.tokens[blockStart:p.index], css_lexer.TEndOfFile, convertTokensOpts{allowImports: true})
 			return css_ast.Rule{Loc: atRange.Loc, Data: &css_ast.RUnknownAt{AtToken: atToken, Prelude: prelude, Block: block}}
@@ -978,11 +994,12 @@ abortRuleParser:
 		}
 
 		// Read the optional block
+		matchingLoc := p.current().Range.Loc
 		if len(names) <= 1 && p.eat(css_lexer.TOpenBrace) {
 			rules := p.parseListOfRules(ruleContext{
 				parseSelectors: true,
 			})
-			p.expect(css_lexer.TCloseBrace)
+			p.expectWithMatchingLoc(css_lexer.TCloseBrace, matchingLoc)
 			return css_ast.Rule{Loc: atRange.Loc, Data: &css_ast.RAtLayer{Names: names, Rules: rules}}
 		}
 
@@ -1016,7 +1033,7 @@ abortRuleParser:
 			//
 			// Instead of implementing all of that for an extremely obscure feature,
 			// CSS namespaces are just explicitly not supported.
-			p.log.Add(logger.Warning, &p.tracker, atRange, "\"@namespace\" rules are not supported")
+			p.log.AddID(logger.MsgID_CSS_UnsupportedAtNamespace, logger.Warning, &p.tracker, atRange, "\"@namespace\" rules are not supported")
 		}
 	}
 
@@ -1064,13 +1081,15 @@ prelude:
 
 	case atRuleDeclarations:
 		// Parse known rules whose blocks always consist of declarations
+		matchingLoc := p.current().Range.Loc
 		p.expect(css_lexer.TOpenBrace)
 		rules := p.parseListOfDeclarations()
-		p.expect(css_lexer.TCloseBrace)
+		p.expectWithMatchingLoc(css_lexer.TCloseBrace, matchingLoc)
 		return css_ast.Rule{Loc: atRange.Loc, Data: &css_ast.RKnownAt{AtToken: atToken, Prelude: prelude, Rules: rules}}
 
 	case atRuleInheritContext:
 		// Parse known rules whose blocks consist of whatever the current context is
+		matchingLoc := p.current().Range.Loc
 		p.expect(css_lexer.TOpenBrace)
 		var rules []css_ast.Rule
 		if context.isDeclarationList {
@@ -1080,15 +1099,16 @@ prelude:
 				parseSelectors: true,
 			})
 		}
-		p.expect(css_lexer.TCloseBrace)
+		p.expectWithMatchingLoc(css_lexer.TCloseBrace, matchingLoc)
 		return css_ast.Rule{Loc: atRange.Loc, Data: &css_ast.RKnownAt{AtToken: atToken, Prelude: prelude, Rules: rules}}
 
 	case atRuleQualifiedOrEmpty:
+		matchingLoc := p.current().Range.Loc
 		if p.eat(css_lexer.TOpenBrace) {
 			rules := p.parseListOfRules(ruleContext{
 				parseSelectors: true,
 			})
-			p.expect(css_lexer.TCloseBrace)
+			p.expectWithMatchingLoc(css_lexer.TCloseBrace, matchingLoc)
 			return css_ast.Rule{Loc: atRange.Loc, Data: &css_ast.RKnownAt{AtToken: atToken, Prelude: prelude, Rules: rules}}
 		}
 		p.expect(css_lexer.TSemicolon)
@@ -1110,7 +1130,7 @@ func (p *parser) expectValidLayerNameIdent() (string, bool) {
 	}
 	switch text {
 	case "initial", "inherit", "unset":
-		p.log.Add(logger.Warning, &p.tracker, r, fmt.Sprintf("%q cannot be used as a layer name", text))
+		p.log.AddID(logger.MsgID_CSS_InvalidAtLayer, logger.Warning, &p.tracker, r, fmt.Sprintf("%q cannot be used as a layer name", text))
 		p.prevError = r.Loc
 		return "", false
 	}
@@ -1182,7 +1202,7 @@ loop:
 		if opts.isInsideCalcFunction && t.Kind.IsNumeric() && len(result) > 0 && result[len(result)-1].Kind.IsNumeric() &&
 			(strings.HasPrefix(token.Text, "+") || strings.HasPrefix(token.Text, "-")) {
 			// "calc(1+2)" and "calc(1-2)" are invalid
-			p.log.Add(logger.Warning, &p.tracker, logger.Range{Loc: t.Range.Loc, Len: 1},
+			p.log.AddID(logger.MsgID_CSS_InvalidCalc, logger.Warning, &p.tracker, logger.Range{Loc: t.Range.Loc, Len: 1},
 				fmt.Sprintf("The %q operator only works if there is whitespace on both sides", token.Text[:1]))
 		}
 
@@ -1199,11 +1219,11 @@ loop:
 			if opts.isInsideCalcFunction && len(tokens) > 0 {
 				if len(result) == 0 || result[len(result)-1].Kind == css_lexer.TComma {
 					// "calc(-(1 + 2))" is invalid
-					p.log.Add(logger.Warning, &p.tracker, t.Range,
+					p.log.AddID(logger.MsgID_CSS_InvalidCalc, logger.Warning, &p.tracker, t.Range,
 						fmt.Sprintf("%q can only be used as an infix operator, not a prefix operator", token.Text))
 				} else if token.Whitespace != css_ast.WhitespaceBefore || tokens[0].Kind != css_lexer.TWhitespace {
 					// "calc(1- 2)" and "calc(1 -(2))" are invalid
-					p.log.Add(logger.Warning, &p.tracker, t.Range,
+					p.log.AddID(logger.MsgID_CSS_InvalidCalc, logger.Warning, &p.tracker, t.Range,
 						fmt.Sprintf("The %q operator only works if there is whitespace on both sides", token.Text))
 				}
 			}
@@ -1471,9 +1491,10 @@ func (p *parser) parseSelectorRuleFrom(preludeStart int, opts parseSelectorOpts)
 			Selectors: list,
 			HasAtNest: opts.atNestRange.Len != 0,
 		}
+		matchingLoc := p.current().Range.Loc
 		if p.expect(css_lexer.TOpenBrace) {
 			selector.Rules = p.parseListOfDeclarations()
-			p.expect(css_lexer.TCloseBrace)
+			p.expectWithMatchingLoc(css_lexer.TCloseBrace, matchingLoc)
 
 			// Minify "@nest" when possible
 			if p.options.MinifySyntax && selector.HasAtNest {
@@ -1529,9 +1550,10 @@ loop:
 		Prelude: p.convertTokens(p.tokens[preludeStart:p.index]),
 	}
 
+	matchingLoc := p.current().Range.Loc
 	if p.eat(css_lexer.TOpenBrace) {
 		qualified.Rules = p.parseListOfDeclarations()
-		p.expect(css_lexer.TCloseBrace)
+		p.expectWithMatchingLoc(css_lexer.TCloseBrace, matchingLoc)
 	} else if !opts.isAlreadyInvalid {
 		p.expect(css_lexer.TOpenBrace)
 	}
@@ -1616,8 +1638,10 @@ stop:
 		if corrected, ok := css_ast.MaybeCorrectDeclarationTypo(keyText); ok {
 			data := p.tracker.MsgData(keyToken.Range, fmt.Sprintf("%q is not a known CSS property", keyText))
 			data.Location.Suggestion = corrected
-			p.log.AddMsg(logger.Msg{Kind: logger.Warning, Data: data,
-				Notes: []logger.MsgData{{Text: fmt.Sprintf("Did you mean %q instead?", corrected)}}})
+			if override, ok := logger.AllowOverride(p.log.Overrides, logger.MsgID_CSS_UnsupportedCSSProperty, logger.Warning); ok {
+				p.log.AddMsg(logger.Msg{Kind: override, Data: data,
+					Notes: []logger.MsgData{{Text: fmt.Sprintf("Did you mean %q instead?", corrected)}}})
+			}
 		}
 	}
 
@@ -1653,10 +1677,11 @@ func (p *parser) parseComponentValue() {
 }
 
 func (p *parser) parseBlock(open css_lexer.T, close css_lexer.T) {
+	matchingLoc := p.current().Range.Loc
 	if p.expect(open) {
 		for !p.eat(close) {
 			if p.peek(css_lexer.TEndOfFile) {
-				p.expect(close)
+				p.expectWithMatchingLoc(close, matchingLoc)
 				return
 			}
 
